@@ -23,7 +23,7 @@ import java.nio.ByteBuffer;
 /** Socket
  *
  * @author Mladen Turk
- * @version $Revision: 706574 $, $Date: 2008-10-21 12:47:24 +0200 (Tue, 21 Oct 2008) $
+ * @version $Id: Socket.java 939309 2010-04-29 13:54:26Z kkolinko $
  */
 
 public class Socket {
@@ -170,6 +170,16 @@ public class Socket {
      *          made the connection request.  This is the socket which should
      *          be used for all future communication.
      */
+    public static native long acceptx(long sock, long pool)
+        throws Exception;
+
+    /**
+     * Accept a new connection request
+     * @param sock The socket we are listening on.
+     * @return  A copy of the socket that is connected to the socket that
+     *          made the connection request.  This is the socket which should
+     *          be used for all future communication.
+     */
     public static native long accept(long sock)
         throws Exception;
 
@@ -473,7 +483,7 @@ public class Socket {
      * @param t Value for the timeout in microseconds.
      * <PRE>
      * t > 0  -- read and write calls return APR_TIMEUP if specified time
-     *           elapsess with no data read or written
+     *           elapses with no data read or written
      * t == 0 -- read and write calls never block
      * t < 0  -- read and write calls block
      * </PRE>
@@ -527,8 +537,8 @@ public class Socket {
         throws Exception;
 
     /**
-     * Private method for geting the socket struct members
-     * @param socket The soocket to use
+     * Private method for getting the socket struct members
+     * @param socket The socket to use
      * @param what Struct member to obtain
      * <PRE>
      * SOCKET_GET_POOL  - The socket pool
@@ -536,7 +546,7 @@ public class Socket {
      * SOCKET_GET_APRS  - APR socket
      * SOCKET_GET_TYPE  - Socket type
      * </PRE>
-     * @return The stucture member address
+     * @return The structure member address
      */
     private static native long get(long socket, int what);
 
@@ -544,7 +554,7 @@ public class Socket {
      * Set internal send ByteBuffer.
      * This function will preset internal Java ByteBuffer for
      * consecutive sendbb calls.
-     * @param thesocket The socket to use
+     * @param sock The socket to use
      * @param buf The ByteBuffer
      */
     public static native void setsbb(long sock, ByteBuffer buf);
@@ -553,7 +563,7 @@ public class Socket {
      * Set internal receive ByteBuffer.
      * This function will preset internal Java ByteBuffer for
      * consecutive revcvbb/recvbbt calls.
-     * @param thesocket The socket to use
+     * @param sock The socket to use
      * @param buf The ByteBuffer
      */
     public static native void setrbb(long sock, ByteBuffer buf);
@@ -563,13 +573,11 @@ public class Socket {
      * @param sock The currently open socket.
      * @param data The user data to associate with the socket.
      * @param key The key to associate with the data.
-     * @param cleanup The cleanup to call when the socket is destroyed.
      */
       public static native int dataSet(long sock, String key, Object data);
 
     /**
      * Return the data associated with the current socket
-     * @param data The user data associated with the socket.
      * @param key The key to associate with the user data.
      * @param sock The currently open socket.
      * @return Data or null in case of error.
